==> kbibtex-0.1.3.tar.bz2: kbibtex-0.1.3/src/bibtexfileexportertoolchain.cpp <==
/***************************************************************************
*   Copyright (C) 2004-2005 by Thomas Fischer                             *
*   fischer@unix-ag.uni-kl.de                                             *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/

#include <qprocess.h>
#include <qwaitcondition.h>
#include <qapplication.h>

#include "bibtexfileexportertoolchain.h"

namespace BibTeX
{

    BibTeXFileExporterToolchain::BibTeXFileExporterToolchain()
            : BibTeXFileExporter()
    {
        wc = new QWaitCondition();
        workingDir = createTempDir();
    }

    BibTeXFileExporterToolchain::~BibTeXFileExporterToolchain()
    {
        delete wc;
        deleteTempDir( workingDir );
    }

    bool BibTeXFileExporterToolchain::runProcesses( const QStringList &progs )
    {
        bool result = TRUE;
        int i = 0;

        emit progress( 0, progs.size() );
        for ( QStringList::ConstIterator it = progs.begin(); result && it != progs.end(); it++ )
        {
            QStringList args = QStringList::split( ' ', *it );
            result &= runProcess( args );
            emit progress( i++, progs.size() );
        }
        return result;
    }

    bool BibTeXFileExporterToolchain::runProcess( const QStringList &args )
    {
        bool result = FALSE;
        QApplication::setOverrideCursor( Qt::waitCursor );

        process = new QProcess( args );
        process->setWorkingDirectory( workingDir );
        connect( process, SIGNAL( processExited() ), this, SLOT( slotProcessExited() ) );
        connect( process, SIGNAL( readyReadStdout() ), this, SLOT( slotReadProcessOutput() ) );
        connect( process, SIGNAL( readyReadStderr() ), this, SLOT( slotReadProcessOutput() ) );

        if ( process->start() )
        {
            qApp->processEvents();
            while ( process->isRunning() )
            {
                wc->wait( 250 );
                qApp->processEvents();
            }

            result = process->normalExit();
        }

        disconnect( process, SIGNAL( readyReadStdout() ), this, SLOT( slotReadProcessOutput() ) );
        disconnect( process, SIGNAL( readyReadStderr() ), this, SLOT( slotReadProcessOutput() ) );
        disconnect( process, SIGNAL( processExited() ), this, SLOT( slotProcessExited() ) );
        delete( process );
        process = NULL;

        QApplication::restoreOverrideCursor();
        return result;
    }

    bool BibTeXFileExporterToolchain::writeFileToIODevice( const QString &filename, QIODevice *device )
    {
        QFile file( filename );
        if ( file.open( IO_ReadOnly ) )
        {
            bool result = TRUE;
            Q_ULONG buffersize = 0x10000;
            Q_LONG amount = 0;
            char* buffer = new char[ buffersize ];
            do
            {
                result = ( ( amount = file.readBlock( buffer, buffersize ) ) > -1 ) && ( device->writeBlock( buffer, amount ) > -1 );
            }
            while ( result && amount > 0 );

            file.close();
            delete buffer;
            return result;
        }
        else
            return FALSE;
    }

    QString BibTeXFileExporterToolchain::createTempDir()
    {
        QString result = QString::null;
        QFile *devrandom = new QFile( "/dev/random" );

        if ( devrandom->open( IO_ReadOnly ) )
        {
            Q_UINT32 randomNumber;
            if ( devrandom->readBlock( ( char* ) & randomNumber, sizeof( randomNumber ) ) > 0 )
            {
                randomNumber |= 0x10000000;
                result = QString( "/tmp/bibtex-%1" ).arg( randomNumber, sizeof( randomNumber ) * 2, 16 );
                qDebug( "Creating temp dir \"%s\"", result.latin1() );
                if ( !QDir().mkdir( result ) )
                    result = QString::null;
            }
            devrandom->close();
        }

        delete devrandom;

        return result;
    }

    void BibTeXFileExporterToolchain::deleteTempDir( const QString& directory )
    {
        QDir dir = QDir( directory );
        QStringList subDirs = dir.entryList( QDir::Dirs );
        for ( QStringList::Iterator it = subDirs.begin(); it != subDirs.end(); it++ )
        {
            if ( ( QString::compare( *it, "." ) != 0 ) && ( QString::compare( *it, ".." ) != 0 ) )
                deleteTempDir( *it );
        }
        QStringList allEntries = dir.entryList( QDir::All );
        for ( QStringList::Iterator it = allEntries.begin(); it != allEntries.end(); it++ )
            dir.remove( *it );

        QDir().rmdir( directory );
        qDebug( "removing dir %s", directory.latin1() );
    }

    void BibTeXFileExporterToolchain::slotProcessExited()
    {
        //         qDebug( "slotProcessExited" );
        wc->wakeAll();
    }

    void BibTeXFileExporterToolchain::cancel()
    {
        if ( process != NULL )
        {
            qDebug( "Canceling process" );
            process->tryTerminate();
            process->kill();
            wc->wakeAll();
        }
    }

    void BibTeXFileExporterToolchain::slotReadProcessOutput()
    {
        /*        if ( process )
                {
                    while ( process->canReadLineStdout() )
                        qDebug( "stdout: %s", process->readLineStdout().latin1() );
                    while ( process->canReadLineStderr() )
                        qDebug( "stderr: %s", process->readLineStderr().latin1() );
                }*/
    }


}

#include "bibtexfileexportertoolchain.moc"

==> kbibtex-0.1.4.tar.bz2: kbibtex-0.1.4/src/bibtexfileexportertoolchain.cpp <==
/***************************************************************************
*   Copyright (C) 2004-2005 by Thomas Fischer                             *
*   fischer@unix-ag.uni-kl.de                                             *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/

#include <qprocess.h>
#include <qwaitcondition.h>
#include <qapplication.h>
#include <qstringlist.h>

#include "bibtexfileexportertoolchain.h"

namespace BibTeX
{

    BibTeXFileExporterToolchain::BibTeXFileExporterToolchain()
            : BibTeXFileExporter(), m_errorLog( NULL )
    {
        wc = new QWaitCondition();
        workingDir = createTempDir();
    }

    BibTeXFileExporterToolchain::~BibTeXFileExporterToolchain()
    {
        delete wc;
        deleteTempDir( workingDir );
    }

    bool BibTeXFileExporterToolchain::runProcesses( const QStringList &progs, QStringList *errorLog )
    {
        bool result = TRUE;
        int i = 0;

        emit progress( 0, progs.size() );
        for ( QStringList::ConstIterator it = progs.begin(); result && it != progs.end(); it++ )
        {
            qApp->processEvents();
            QStringList args = QStringList::split( ' ', *it );
            result &= runProcess( args, errorLog );
            emit progress( i++, progs.size() );
        }
        qApp->processEvents();
        return result;
    }

    bool BibTeXFileExporterToolchain::runProcess( const QStringList &args, QStringList *errorLog )
    {
        bool result = FALSE;
        QApplication::setOverrideCursor( Qt::waitCursor );

        process = new QProcess( args );
        process->setWorkingDirectory( workingDir );
        connect( process, SIGNAL( processExited() ), this, SLOT( slotProcessExited() ) );
        connect( process, SIGNAL( readyReadStdout() ), this, SLOT( slotReadProcessOutput() ) );
        connect( process, SIGNAL( readyReadStderr() ), this, SLOT( slotReadProcessOutput() ) );

        if ( process->start() )
        {
            m_errorLog = errorLog;
            int counter = 0;
            qApp->processEvents();
            while ( process->isRunning() )
            {
                wc->wait( 250 );
                qApp->processEvents();

                counter++;
                if ( counter > 400 )
                    process->tryTerminate();
            }

            result = process->normalExit() && counter < 400;
        }

        disconnect( process, SIGNAL( readyReadStdout() ), this, SLOT( slotReadProcessOutput() ) );
        disconnect( process, SIGNAL( readyReadStderr() ), this, SLOT( slotReadProcessOutput() ) );
        disconnect( process, SIGNAL( processExited() ), this, SLOT( slotProcessExited() ) );
        delete( process );
        process = NULL;

        QApplication::restoreOverrideCursor();
        return result;
    }

    bool BibTeXFileExporterToolchain::writeFileToIODevice( const QString &filename, QIODevice *device )
    {
        QFile file( filename );
        if ( file.open( IO_ReadOnly ) )
        {
            bool result = TRUE;
            Q_ULONG buffersize = 0x10000;
            Q_LONG amount = 0;
            char* buffer = new char[ buffersize ];
            do
            {
                result = ( ( amount = file.readBlock( buffer, buffersize ) ) > -1 ) && ( device->writeBlock( buffer, amount ) > -1 );
            }
            while ( result && amount > 0 );

            file.close();
            delete[] buffer;
            return result;
        }
        else
            return FALSE;
    }

    QString BibTeXFileExporterToolchain::createTempDir()
    {
        QString result = QString::null;
        QFile *devrandom = new QFile( "/dev/random" );

        if ( devrandom->open( IO_ReadOnly ) )
        {
            Q_UINT32 randomNumber;
            if ( devrandom->readBlock( ( char* ) & randomNumber, sizeof( randomNumber ) ) > 0 )
            {
                randomNumber |= 0x10000000;
                result = QString( "/tmp/bibtex-%1" ).arg( randomNumber, sizeof( randomNumber ) * 2, 16 );
                if ( !QDir().mkdir( result ) )
                    result = QString::null;
            }
            devrandom->close();
        }

        delete devrandom;

        return result;
    }

    void BibTeXFileExporterToolchain::deleteTempDir( const QString& directory )
    {
        QDir dir = QDir( directory );
        QStringList subDirs = dir.entryList( QDir::Dirs );
        for ( QStringList::Iterator it = subDirs.begin(); it != subDirs.end(); it++ )
        {
            if ( ( QString::compare( *it, "." ) != 0 ) && ( QString::compare( *it, ".." ) != 0 ) )
                deleteTempDir( *it );
        }
        QStringList allEntries = dir.entryList( QDir::All );
        for ( QStringList::Iterator it = allEntries.begin(); it != allEntries.end(); it++ )
            dir.remove( *it );

        QDir().rmdir( directory );
    }

    void BibTeXFileExporterToolchain::slotProcessExited()
    {
        wc->wakeAll();
    }

    void BibTeXFileExporterToolchain::cancel()
    {
        if ( process != NULL )
        {
            qDebug( "Canceling process" );
            process->tryTerminate();
            process->kill();
            wc->wakeAll();
        }
    }

    void BibTeXFileExporterToolchain::slotReadProcessOutput()
    {
        if ( process )
        {
            while ( process->canReadLineStdout() )
            {
                QString line = process->readLineStdout();
                if ( m_errorLog != NULL )
                    m_errorLog->append( line );
            }
            while ( process->canReadLineStderr() )
            {
                QString line = process->readLineStderr();
                if ( m_errorLog != NULL )
                    m_errorLog->append( line );
            }
        }
    }


}

#include "bibtexfileexportertoolchain.moc"
